# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

from collections import defaultdict
import numpy as np
from ..units import (
    allclose, default_units, get_derived_unit, is_unitless, linspace,
    SI_base_registry,
    to_unitless,
    unit_of,
    unit_registry_to_human_readable,
    unit_registry_from_human_readable,
)

metre = default_units.metre
second = default_units.second
hour = default_units.hour
dm = default_units.decimetre
mole = default_units.mole
kilogram = default_units.kilogram
ampere = default_units.ampere
kelvin = default_units.kelvin
candela = default_units.candela
molar = mole / dm**3


def test_default_units():
    # In addition to the above, we guarantee that theses are available
    default_units.joule
    default_units.gray
    default_units.eV
    default_units.MeV
    default_units.metre
    default_units.decimetre
    default_units.centimetre
    default_units.micrometre
    default_units.nanometre
    default_units.gram
    default_units.molar
    default_units.hour
    default_units.perMolar_perSecond
    default_units.per100eV
    default_units.umol
    default_units.umol_per_J


def test_allclose():
    a = np.linspace(2, 3)*second
    b = np.linspace(2/3600., 3/3600.)*hour
    assert allclose(a, b)
    assert allclose([3600*second, 2*metre/hour], [1*hour, 2/3600*metre/second])


def test_is_unitless():
    assert not is_unitless(1*second)
    assert is_unitless(1)


def test_unit_of():
    assert unit_of(0.1*metre/second) == metre/second
    assert unit_of(7) == 1


def test_to_unitless():
    vals = [1.0*dm, 2.0*dm]
    result = to_unitless(vals, metre)
    assert result[0] == 0.1
    assert result[1] == 0.2

    vals = [1.0, 2.0]*dm
    result = to_unitless(vals, metre)
    assert result[0] == 0.1
    assert result[1] == 0.2

    length_unit = 1000*metre
    result = to_unitless(1.0*metre, length_unit)
    assert abs(result - 1e-3) < 1e-12

    amount_unit = 1e-9  # nano
    assert abs(to_unitless(1.0, amount_unit) - 1e9) < 1e-6

    assert abs(to_unitless(3/(second*molar),
                           metre**3/mole/second) - 3e-3) < 1e-12


def test_linspace():
    ls = linspace(2*second, 3*second)
    assert abs(to_unitless(ls[0], hour) - 2/3600.) < 1e-15


def test_get_derived_unit():
    registry = SI_base_registry.copy()
    registry['length'] = 1e-1*registry['length']
    conc_unit = get_derived_unit(registry, 'concentration')
    assert abs(conc_unit - 1*mole/(dm**3)) < 1e-12*mole/(dm**3)

    registry = defaultdict(lambda: 1)
    registry['amount'] = 1e-9  # nano
    assert abs(to_unitless(1.0, get_derived_unit(
        registry, 'concentration')) - 1e9) < 1e-6


def test_unit_registry_to_human_readable():
    # Not as much human readable as JSON serializable...
    d = defaultdict(lambda: 1)
    assert unit_registry_to_human_readable(d) == dict(
        (x, (1, 1)) for x in SI_base_registry.keys())

    ur = {
        'length': 1e3*metre,
        'mass': 1e-2*kilogram,
        'time': 1e4*second,
        'current': 1e-1*ampere,
        'temperature': 1e1*kelvin,
        'luminous_intensity': 1e-3*candela,
        'amount': 1e4*mole
    }
    assert unit_registry_to_human_readable(ur) == {
        'length': (1e3, 'm'),
        'mass': (1e-2, 'kg'),
        'time': (1e4, 's'),
        'current': (1e-1, 'A'),
        'temperature': (1e1, 'K'),
        'luminous_intensity': (1e-3, 'cd'),
        'amount': (1e4, 'mol')
    }
    assert unit_registry_to_human_readable(ur) != {
        'length': (1e2, 'm'),
        'mass': (1e-2, 'kg'),
        'time': (1e4, 's'),
        'current': (1e-1, 'A'),
        'temperature': (1e1, 'K'),
        'luminous_intensity': (1e-3, 'cd'),
        'amount': (1e4, 'mol')
    }


def test_unit_registry_from_human_readable():
    hr = unit_registry_to_human_readable(defaultdict(lambda: 1))
    assert hr == dict((x, (1, 1)) for x in SI_base_registry.keys())
    ur = unit_registry_from_human_readable(hr)
    assert ur == dict((x, 1) for x in SI_base_registry.keys())

    hr = unit_registry_to_human_readable(SI_base_registry)
    assert hr == {
        'length': (1.0, 'm'),
        'mass': (1.0, 'kg'),
        'time': (1.0, 's'),
        'current': (1.0, 'A'),
        'temperature': (1.0, 'K'),
        'luminous_intensity': (1.0, 'cd'),
        'amount': (1.0, 'mol')
    }
    ur = unit_registry_from_human_readable(hr)
    assert ur == SI_base_registry

    ur = unit_registry_from_human_readable({
        'length': (1.0, 'm'),
        'mass': (1.0, 'kg'),
        'time': (1.0, 's'),
        'current': (1.0, 'A'),
        'temperature': (1.0, 'K'),
        'luminous_intensity': (1.0, 'cd'),
        'amount': (1.0, 'mol')
    })
    assert ur == {
        'length': metre,
        'mass': kilogram,
        'time': second,
        'current': ampere,
        'temperature': kelvin,
        'luminous_intensity': candela,
        'amount': mole
    }

    ur = unit_registry_from_human_readable({
        'length': (1e3, 'm'),
        'mass': (1e-2, 'kg'),
        'time': (1e4, 's'),
        'current': (1e-1, 'A'),
        'temperature': (1e1, 'K'),
        'luminous_intensity': (1e-3, 'cd'),
        'amount': (1e4, 'mol')
    })
    assert ur == {
        'length': 1e3*metre,
        'mass': 1e-2*kilogram,
        'time': 1e4*second,
        'current': 1e-1*ampere,
        'temperature': 1e1*kelvin,
        'luminous_intensity': 1e-3*candela,
        'amount': 1e4*mole
    }

    assert ur != {
        'length': 1e2*metre,
        'mass': 1e-3*kilogram,
        'time': 1e2*second,
        'current': 1e-2*ampere,
        'temperature': 1e0*kelvin,
        'luminous_intensity': 1e-2*candela,
        'amount': 1e3*mole
    }



import warnings

from chempy.units import allclose
from ..water_diffusivity_holz_2000 import water_self_diffusion_coefficient as \
    w_sd


def test_water_self_diffusion_coefficient():
    warnings.filterwarnings("error")
    assert abs(w_sd(273.15 + 0.) - 1.099e-9) < 0.027e-9
    assert abs(w_sd(273.15 + 4.) - 1.261e-9) < 0.011e-9
    assert abs(w_sd(273.15 + 10) - 1.525e-9) < 0.007e-9
    assert abs(w_sd(273.15 + 15) - 1.765e-9) < 0.006e-9
    assert abs(w_sd(273.15 + 20) - 2.023e-9) < 0.001e-9
    assert abs(w_sd(273.15 + 25) - 2.299e-9) < 0.001e-9
    assert abs(w_sd(273.15 + 30) - 2.594e-9) < 0.001e-9
    assert abs(w_sd(273.15 + 35) - 2.907e-9) < 0.004e-9

    try:
        w_sd(1)
    except UserWarning:
        pass  # good warning raised
    else:
        raise
    warnings.resetwarnings()

    try:
        import quantities as pq
        import numpy as np
        unit = pq.m**2/pq.s
        assert allclose(1e9*w_sd(298.15*pq.K, units=pq),
                        2.299*unit, rtol=1e-3, atol=1e-8*unit)
        assert allclose(1e9*w_sd(np.linspace(297, 299)*pq.K, units=pq),
                        2.299*pq.m**2/pq.s, rtol=5e-2, atol=1e-2*unit)
    except ImportError:
        pass



import warnings

from chempy.units import allclose

from ..water_permittivity_bradley_pitzer_1979 import water_permittivity


def test_water_permittivity():
    warnings.filterwarnings("error")
    abs(water_permittivity(273.15 + 0) - 80) < 1.0
    abs(water_permittivity(273.15 + 20) - 80.1) < 0.2
    abs(water_permittivity(273.15 + 100) - 55.3) < 0.5

    try:
        water_permittivity(1)
    except UserWarning:
        pass  # good: warning raised
    else:
        raise
    warnings.resetwarnings()

    try:
        import quantities as pq
        import numpy as np
        assert allclose(water_permittivity(
            298.15*pq.K, 1*pq.bar,
            units=pq), 78.38436874203077)
        assert allclose(water_permittivity(
            np.linspace(297.5, 298.65)*pq.K, 1*pq.bar,
            units=pq), 78, rtol=1e-2, atol=1e-2)
    except ImportError:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
import shutil
from setuptools import setup

pkg_name = "chempy"

RELEASE_VERSION = os.environ.get('CHEMPY_RELEASE_VERSION', '')

# http://conda.pydata.org/docs/build.html#environment-variables-set-during-the-build-process
if os.environ.get('CONDA_BUILD', '0') == '1':
    try:
        RELEASE_VERSION = 'v' + io.open(
            '__conda_version__.txt', 'rt', encoding='utf-8'
        ).readline().rstrip()
    except IOError:
        pass


def _path_under_setup(*args):
    return os.path.join(os.path.dirname(__file__), *args)

release_py_path = _path_under_setup(pkg_name, '_release.py')

if (len(RELEASE_VERSION) > 1 and RELEASE_VERSION[0] == 'v'):
    TAGGED_RELEASE = True
    __version__ = RELEASE_VERSION[1:]
else:
    TAGGED_RELEASE = False
    # read __version__ attribute from _release.py:
    exec(io.open(release_py_path, encoding='utf-8').read())


submodules = [
    'chempy.kinetics',
    'chempy.properties',
    'chempy.util',
]

tests = [
    'chempy.tests',
    'chempy.kinetics.tests',
    'chempy.properties.tests',
    'chempy.util.tests',
]

classifiers = [
    "Development Status :: 3 - Alpha",
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Scientific/Engineering',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
]

with io.open(_path_under_setup(pkg_name, '__init__.py'), 'rt',
             encoding='utf-8') as f:
    short_description = f.read().split('"""')[1].split('\n')[1]
assert 10 < len(short_description) < 255
long_description = open(_path_under_setup('README.rst')).read()
assert len(long_description) > 100


setup_kwargs = {
    'name': pkg_name,
    'version': __version__,
    'description': short_description,
    'long_description': long_description,
    'author': 'Björn Dahlgren',
    'author_email': 'bjodah@DELETEMEgmail.com',
    'license': 'BSD',
    'keywords': ("chemistry", "water properties", "physical chemistry"),
    'url': 'https://github.com/bjodah/' + pkg_name,
    'packages': [pkg_name] + submodules + tests,
    'classifiers': classifiers,
    'install_requires': [
        'numpy>1.7', 'scipy>=0.16.1', 'matplotlib>=1.3.1',
        'sympy>=0.7.6.1', 'quantities>=0.11.1', 'pyneqsys>=0.3.0',
        'pyodesys>=0.5.0', 'pyparsing>=2.0.3',
        # 'dot2tex>=2.9.0'
    ],
    'extras_require': {
        'all': ['argh', 'pycvodes', 'pygslodeiv2', 'pyodeint', 'pykinsol',
                'pytest>=2.8.1', 'pytest-pep8>=1.0.6']}
}

if __name__ == '__main__':
    try:
        if TAGGED_RELEASE:
            # Same commit should generate different sdist
            # depending on tagged version (set CHEMPY_RELEASE_VERSION)
            # this will ensure source distributions contain the correct version
            shutil.move(release_py_path, release_py_path+'__temp__')
            open(release_py_path, 'wt').write(
                "__version__ = '{}'\n".format(__version__))
        setup(**setup_kwargs)
    finally:
        if TAGGED_RELEASE:
            shutil.move(release_py_path+'__temp__', release_py_path)



# -*- coding: utf-8 -*-

import warnings

try:
    from numpy import any as _any
except ImportError:
    _any = any

# Parameters from paper (SI-units):

gamma = 2.063
dgamma = 0.051
D0 = 1.635e-8
TS = 215.05
dD0 = 2.242e-11
dTS = 1.2
low_t_bound = 273.15  # 0 deg C, (m.p. at ambient pressure)
high_t_bound = 373.15  # 100 deg C, (b.p. at ambient pressure)


def water_self_diffusion_coefficient(T=None, units=None, warn=True,
                                     err_mult=None):
    """
    Temperature-dependent self-diffusion coefficient of water.

    Parameters
    ----------
    T: float
        Temperature (default: in Kelvin)
    units: object (optional)
        object with attributes: Kelvin, meter, kilogram
    warn: bool (default: True)
        Emit UserWarning when outside temperature range.
    err_mult: length 2 array_like (default: None)
        Perturb paramaters D0 and TS with err_mult[0]*dD0 and
        err_mult[1]*dTS respectively, where dD0 and dTS are the
        reported uncertainties in the fitted paramters. Useful
        for estimating error in diffusion coefficient.

    References
    ----------
    Temperature-dependent self-diffusion coefficients of water and six selected
        molecular liquids for calibration in accurate 1H NMR PFG measurements
        Manfred Holz, Stefan R. Heila, Antonio Saccob;
        Phys. Chem. Chem. Phys., 2000,2, 4740-4742
        http://pubs.rsc.org/en/Content/ArticleLanding/2000/CP/b005319h
        DOI: 10.1039/B005319H
    """
    if units is None:
        K = 1
        m = 1
        s = 1
    else:
        K = units.Kelvin
        m = units.meter
        s = units.second
    if T is None:
        T = 298.15*K
    _D0 = D0 * m**2 * s**-1
    _TS = TS * K
    if err_mult is not None:
        _dD0 = dD0 * m**2 * s**-1
        _dTS = dTS * K
        _D0 += err_mult[0]*_dD0
        _TS += err_mult[1]*_dTS
    if warn and (_any(T < low_t_bound*K) or _any(T > high_t_bound*K)):
        warnings.warn("Temperature is outside range (0-100 degC)")
    return _D0*((T/_TS) - 1)**gamma

# generated at doi2bib.org:
bibtex = """
@article{Holz2000,
  doi = {10.1039/b005319h},
  url = {http://dx.doi.org/10.1039/B005319H},
  year  = {2000},
  publisher = {Royal Society of Chemistry ({RSC})},
  volume = {2},
  number = {20},
  pages = {4740--4742},
  author = {Manfred Holz and Stefan R. Heil and Antonio Sacco},
  title = {Temperature-dependent self-diffusion coefficients of water and six
           selected molecular liquids for calibration in accurate 1H {NMR}
           {PFG} measurements},
  journal = {Phys. Chem. Chem. Phys.}
}
"""



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function


class AttributeDict(object):
    def __init__(self, d):
        self.__dict__.update(d)


def get_constant_symbols(Symbol=None):
    if Symbol is None:
        from sympy import Symbol
    consts = [
        ('Faraday_constant', 'F'),
        ('Avogadro_constant', 'N_A'),
        ('vacuum_permittivity', 'epsilon_0'),
        ('Boltzmann_constant', 'k_B'),
        ('pi', 'pi'),
        ('molar_gas_constant', 'R'),
    ]
    return AttributeDict({k: Symbol(v) for k, v in consts})



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

from collections import OrderedDict

import numpy as np

from ..stoich import (
    get_coeff_mtx, decompose_yields,
)


def test_get_coeff_mtx():
    r = [
        ({'A': 1},       {'B': 1}),
        ({'A': 1, 'B': 1}, {'C': 2})
    ]
    A = get_coeff_mtx('ABC', r)
    Aref = np.array([
        [-1, -1],
        [1, -1],
        [0,  2]
    ])
    assert np.allclose(A, Aref)


def test_decompose_yields_1():
    gamma_yields = OrderedDict([
        ('OH-', 0.5),
        ('H2O2', 0.7),
        ('OH', 2.7),
        ('H2', 0.45),
        ('H', 0.66),
        ('H+', 3.1),
        ('HO2', 0.02),
        ('e-aq', 2.6),
        # H2O: -4.64
    ])

    stoichs = [
        ({'H2O': 1}, {'H+': 1, 'OH-': 1}),
        ({'H2O': 1}, {'H+': 1, 'e-aq': 1, 'OH': 1}),
        ({'H2O': 2}, {'H': 2, 'H2O2': 1}),
        ({'H2O': 2}, {'H2': 1, 'H2O2': 1}),
        ({'H2O': 2}, {'H2': 1, 'OH': 2}),
        ({'H2O': 4}, {'H2': 3, 'HO2': 2}),
    ]

    k = decompose_yields(gamma_yields, stoichs)
    k_ref = [0.5, 2.6, 0.33, 0.37, 0.05, 0.01]

    assert np.allclose(k, k_ref)

    G_H2O = sum(-reac['H2O']*k[i] for
                i, (reac, prod) in enumerate(stoichs))

    assert abs(G_H2O+4.64) < 1e-3


def test_decompose_yields_2():
    yields = OrderedDict([
        ('B', 3.0),
        ('C', 24.0)
    ])
    stoichs = [
        ({'A': 2}, {'B': 1, 'C': 1}),
        ({'A': 1}, {'C': 3})
    ]
    k = decompose_yields(yields, stoichs)
    k_ref = [3, 7]

    rtol = 1e-12
    for a, b in zip(k, k_ref):
        assert abs(a-b) < abs(a*rtol)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)


from ..arithmeticdict import ArithmeticDict


def test_ArithmeticDict():
    d1 = ArithmeticDict(float, [('a', 1.0), ('b', 2.0)])
    d2 = ArithmeticDict(float, [('c', 5.0), ('b', 3.0)])
    d3 = d1+d2
    assert d3['a'] == 1.0
    assert d3['b'] == 5.0
    assert d3['c'] == 5.0
    d3 += {'c': 1.0}
    assert d3['a'] == 1.0
    assert d3['b'] == 5.0
    assert d3['c'] == 6.0
    d4 = {'a': 7.0} + d1
    assert d4['a'] == 8.0
    d5 = d1 + {'a': 9.0}
    assert d5['a'] == 10.0

    d6 = d1 - d2
    assert d6 == {'a': 1.0, 'b': -1.0, 'c': -5.0}
    d6 -= d3
    assert d6 == {'b': -6.0, 'c': -11.0}


def test_ArithmeticDict_add():
    d1 = ArithmeticDict(int, [('a', 1), ('b', 2)])
    d2 = d1+3
    assert d2['a'] == 4
    assert d2['b'] == 5
    d2 = 3+d1
    assert d2['a'] == 4
    assert d2['b'] == 5
    assert d2['c'] == 0
    d3 = d1 + d2
    assert d3['a'] == 5
    assert d3['b'] == 7
    assert d3['c'] == 0


def test_ArithmeticDict_iadd():
    d1 = ArithmeticDict(int, [('a', 1), ('b', 2)])
    d1 += 3
    assert d1['a'] == 4
    assert d1['b'] == 5
    assert d1['c'] == 0


def test_ArithmeticDict_sub():
    d1 = ArithmeticDict(int, [('a', 1), ('b', 2)])
    d2 = d1-7
    assert d2['a'] == -6
    assert d2['b'] == -5
    d2 = 3-d1
    assert d2['a'] == 2
    assert d2['b'] == 1
    assert d2['c'] == 0
    d3 = d1 - d2
    assert d3['a'] == -1
    assert d3['b'] == 1


def test_ArithmeticDict_isub():
    d1 = ArithmeticDict(int, [('a', 1), ('b', 2)])
    d1 -= 7
    assert d1['a'] == -6
    assert d1['b'] == -5
    assert d1['c'] == 0


def test_ArithmeticDict_mul():
    d1 = ArithmeticDict(int, [('a', 1), ('b', 2)])
    d2 = d1*3
    assert d2['a'] == 3
    assert d2['b'] == 6
    d2 = 3*d1
    assert d2['a'] == 3
    assert d2['b'] == 6
    assert d2['c'] == 0
    d3 = d1 * d2
    assert d3['a'] == 3
    assert d3['b'] == 12
    assert d3['c'] == 0


def test_ArithmeticDict_imul():
    d1 = ArithmeticDict(int, [('a', 1), ('b', 2)])
    d1 *= 3
    assert d1['a'] == 3
    assert d1['b'] == 6
    assert d1['c'] == 0


def test_ArithmeticDict_div():
    d1 = ArithmeticDict(int, [('a', 6), ('b', 9)])

    d2 = d1/3
    assert d2['a'] == 2
    assert d2['b'] == 3

    d2 = d1.copy() / 3
    assert d2['a'] == 2
    assert d2['b'] == 3

    d2 = 54/d1
    assert d2['a'] == 9
    assert d2['b'] == 6
    assert d2['c'] == 0

    d1['c'] = 1
    d3 = (6*d2)/d1
    assert d3['a'] == 9
    assert d3['b'] == 4
    assert d3['c'] == 0


def test_ArithmeticDict_idiv():
    d1 = ArithmeticDict(int, [('a', 6), ('b', 9)])
    d1 /= 3
    assert d1['a'] == 2
    assert d1['b'] == 3
    assert d1['c'] == 0


def test_ArithmeticDict_div_float():
    d = ArithmeticDict(float)
    d['a'] = 6.0
    d['b'] = 9.0
    t = d/3.0
    assert t['a'] == 2.0
    assert t['b'] == 3.0



# -*- coding: utf-8 -*-
"""
General utilities and exceptions.
"""
from __future__ import (absolute_import, division, print_function)

from collections import namedtuple, Mapping
import os
import warnings

from .. import __url__
from .deprecation import Deprecation


def defaultnamedtuple(typename, field_names, defaults=()):
    """ Generates a new subclass of tuple with default values.

    Parameters
    ----------
    typename : string
        The name of the class.
    field_names : str or iterable
        An iterable of splitable string.
    defaults : iterable
        Default values for ``field_names``, counting ``[-len(defaults):]``.

    Examples
    --------
    >>> Body = defaultnamedtuple('Body', 'x y z density', (1.0,))
    >>> Body.__doc__
    'Body(x, y, z, density)'
    >>> b = Body(10, z=3, y=5)
    >>> b._asdict()
    OrderedDict([('x', 10), ('y', 5), ('z', 3), ('density', 1.0)])

    Returns
    -------
    A new tuple subclass named ``typename``

    """
    Tuple = namedtuple(typename, field_names)
    Tuple.__new__.__defaults__ = (None,) * len(Tuple._fields)
    if isinstance(defaults, Mapping):
        Tuple.__new__.__defaults__ = tuple(Tuple(**defaults))
    else:
        nmissing = len(Tuple._fields) - len(defaults)
        defaults = (None,)*nmissing + tuple(defaults)
        Tuple.__new__.__defaults__ = tuple(Tuple(*defaults))
    return Tuple


class NoConvergence(Exception):
    pass  # why is this not in the Python standard library!?


class ChemPyDeprecationWarning(DeprecationWarning):
    pass


def deprecated(*args, **kwargs):
    """ Helper to :class:`Deprecation` for using ChemPyDeprecationWarning. """
    return Deprecation(*args, issues_url=__url__ + '/issues/%s/',
                       warning=ChemPyDeprecationWarning, **kwargs)

warnings.simplefilter(os.environ.get('CHEMPY_DEPRECATION_FILTER', 'once'),
                      ChemPyDeprecationWarning)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function)

from ..sulfuric_acid_density_myhre_1998 import (
    sulfuric_acid_density, density_from_concentration
)


def test_sulfuric_acid_density():
    rho = sulfuric_acid_density(.1, 298)
    assert abs(1063.8 - rho) < .1


def test_density_from_concentration():
    rho = density_from_concentration(1000)
    assert abs(1058.5 - rho) < .1



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


def ionic_strength(molalities, charges, b0=1):
    """ Calculates the ionic strength

    Parameters
    ----------
    molalities: float
        optionally with unit (amount / mass)
    charges: iterable of integers
        charge of respective ion
    b0: float
        reference molality, optionally with unit (amount / mass)
        by IUPAC defines it as 1 mol/kg. (default: 1)

    Examples
    --------
    >>> ionic_strength([1e-3, 3e-3], [3, -1]) == .5 * (9 + 3) * 1e-3
    True

    """
    tot = 0
    if len(molalities) != len(charges):
        raise ValueError("molalities and charges of different lengths")
    for b, z in zip(molalities, charges):
        tot += (b/b0)*z**2
    return tot/2


class _ActivityProductBase(object):
    """ Baseclass for activity products """

    def __init__(self, stoich, *args):
        self.stoich = stoich
        self.args = args

    def __call__(self, c):
        pass
